#include "entity.h"

#include <cmath>

#include "res.h"
#include "config.h"


namespace el {



Entity::Entity() {
	remove=false;
	collision=0;
	rg=NULL;
	movement=NULL;
	movementdata=0;
	collisionid=0;
	through=false;
	img1=0;
	img2=0;

	rad1=2;
	rad2=4;
}


void Entity::setmode(entitymode i) {

	switch (i) {
		case MSHOT:
			img1=MSHOT;
			rad1=2;
			img2=2;
			rad1=6;
			collisionid=1;
			break;
		case MBLANK1:
			img1=MBLANK1;
			rad1=66;
			collisionid=2;
			break;
		default:
			std::cout << "entity setmode invalid " << i << std::endl;
			break;
	}
}


SDL_Rect epos(float x,float y,int ra) {
	SDL_Rect r;

	r.x=((int)floorf(x))-ra/2;
	r.y=((int)floorf(y))-ra/2;
	r.w=ra;
	r.h=ra;
	return r;
}

void Entity::draw(SDL_Surface *s) {
	if (remove) return;
	SDL_Rect rect;
	
	if (collision>0) {
		rect=epos(x,y,32);
		
		
		SDL_SetClipRect(s,&rect);
		rect.x+=(collision-5)*32;
		SDL_BlitSurface(res::get(3),NULL,s,&rect);


		collision--;
		if (collision==1) { 
			remove=true; 
			return; 
		}
		return;
	}


	if (img2!=0) {
		rect=epos(x,y,rad2);
		SDL_SetClipRect(s,&rect);
		SDL_BlitSurface(res::get(img2),NULL,s,NULL);
	}


	rect=epos(x,y,rad1);

	SDL_SetClipRect(s,&rect);

	switch (img1) {
		case MBLANK1:
			SDL_FillRect(s,NULL,SDL_MapRGB(s->format,0,0,0));
			break;
		default:
			SDL_BlitSurface(res::get(img1),NULL,s,NULL);
			break;

	}
}
void Entity::tick(int ticks) {
	if (collision>0) return;
	if (((int)y)>=SY) { remove=true; return; }
	if (((int)y)<STATUSY) { remove=true; return; }
	if (((int)x)>=SX+100) { remove=true; return; }
	if (((int)x)<0) { 
		through=true;
		remove=true; 
		return; 
	}
	if (remove) return;
	if (movement==NULL) return;
	movement(this);
}
void Entity::clone(Entity *e) {
	x=e->x;
	y=e->y;
	img1=e->img1;
	img2=e->img2;
	rad1=e->rad1;
	rad2=e->rad2;

	collisionid=e->collisionid;

	rg=e->rg;
	movement=e->movement;
	movementdata=e->movementdata;
}


std::ostream& operator<<(std::ostream&s, Entity sk) {
	return s;
}

} //namespace
