#include "SDL.h"
#include "SDL_image.h"
#ifdef USE_TTF
#include "SDL_ttf.h"
#else 
#include "font.h"
#endif

#include <iostream>
#include <vector>
#include <string>
#include <sstream>


#include "config.h"
#include "res.h"
#include "entity.h"
#include "random.h"
#include "movement.h"
#include "formation.h"
#include "player.h"
#include "level.h"




namespace el {

SDL_Surface *screen=NULL;
#ifdef USE_TTF
TTF_Font *font=NULL;
#endif
std::vector <Entity> entities;
std::vector <Level> levels;
Random rg;
Player player;
int round;
unsigned int lastwave=0;



void start() {
	SDL_SetClipRect(screen,NULL);
	SDL_FillRect(screen,NULL,SDL_MapRGB(screen->format,0,0,0));
	SDL_Flip(screen);

	entities.resize(0);
	player.score=0;
	player.lives=10;
	player.livelost=false;
	player.gameover=false;
	player.waves=0;

	round=0;
	lastwave=0;


	player.x=(float)10;
	player.y=(float)SY/2-25;
	player.level=0;

}


void setup() {
	Level l;

	l.id=0;
	l.wavedelay=5000;
	l.waves=6;
	l.movements.push_back(mov_random);
	l.formations.push_back(form_behind);
	l.images.push_back(RESERVED+0);
	l.images.push_back(RESERVED+1);
	l.images.push_back(RESERVED+2);
	levels.push_back(l);
	l.reset();


	l.id=1;
	l.wavedelay=4500;
	l.waves=7;
	l.movements.push_back(mov_random);
	l.movements.push_back(mov_sin);
	l.formations.push_back(form_para);
	l.formations.push_back(form_behind);
	l.images.push_back(RESERVED+2);
	l.images.push_back(RESERVED+3);
	l.images.push_back(RESERVED+4);
	levels.push_back(l);
	l.reset();

	l.id=2;
	l.wavedelay=4000;
	l.waves=8;
	l.movements.push_back(mov_random);
	l.movements.push_back(mov_sin);
	l.movements.push_back(mov_skewbackdown);
	l.movements.push_back(mov_skewbackup);
	l.formations.push_back(form_para);
	l.formations.push_back(form_behind);
	l.images.push_back(RESERVED+4);
	l.images.push_back(RESERVED+5);
	l.images.push_back(RESERVED+6);
	levels.push_back(l);
	l.reset();

	l.id=3;
	l.wavedelay=4000;
	l.waves=8;
	l.movements.push_back(mov_sincos);
	l.movements.push_back(mov_skewdown);
	l.movements.push_back(mov_skewup);
	l.movements.push_back(mov_diagonalrandom);
	l.formations.push_back(form_para);
	l.formations.push_back(form_behind);
	l.images.push_back(RESERVED+4);
	l.images.push_back(RESERVED+5);
	l.images.push_back(RESERVED+6);
	levels.push_back(l);
	l.reset();

	l.id=4;
	l.wavedelay=4000;
	l.waves=8;
	l.movements.push_back(left);
	l.movements.push_back(mov_skewdown);
	l.movements.push_back(mov_skewup);
	l.movements.push_back(mov_diagonalrandom);
	l.formations.push_back(form_para);
	l.formations.push_back(form_behind);
	l.images.push_back(RESERVED+4);
	l.images.push_back(RESERVED+2);
	l.images.push_back(RESERVED+6);
	levels.push_back(l);
	l.reset();
}



Level getlevel(int l) {
	if (levels.size()==0) { std::cerr << "no levels! " << std::endl; }
	if (l<0) { std::cerr << " level <0 " << std::endl; }
	if (l>=(int)levels.size()) {
		return levels[levels.size()-1];
	}
	return levels[l];
}

void add_entities(bool extra=false) {
	Entity e;

	Level level;
	level=getlevel(player.level);



	if (!extra) {
		if (rg.get(35)!=2) return;
		if (rg.get(85)==3) {
			e.x=(float)300+rg.get(300);
			e.rg=Random::instance();
			e.img1=level.getimage();
			e.img2=level.getimage();

			switch (rg.get(4)) {
				case 0: e.y=0; e.movement=down; break;
				case 1: e.y=479; e.movement=up; break;
				case 2: e.y=0; e.movement=downleft; break;
				case 3: e.y=479; e.movement=upleft; break;
			}
			entities.push_back(e);

			form_parax(&e,&entities);
		}


	
		if (lastwave+level.wavedelay>SDL_GetTicks()) return;
		player.waves++;
		if (player.waves>level.waves) {
			player.level++;
			player.waves=0;
			level=getlevel(player.level);
		}
		lastwave=SDL_GetTicks();
	}


	e.y=(float)STATUSY+rg.get(SY-STATUSY); e.x=(float)SX-1;
	e.rg=Random::instance();
	e.img1=level.getimage();
	e.img2=level.getimage();
	e.movement=level.getmovement();
	entities.push_back(e);

	formation_fun ff;

	ff=level.getformation();
	ff(&e,&entities);

}



void fire(entitymode i) {
	Entity e;
	if (player.gameover) return;

	e.y=player.y+25; 
	e.x=player.x;
	e.rg=Random::instance();
	e.movement=right;

	e.setmode(i);

	entities.push_back(e);
}



void status() {
	SDL_Rect rect;
	rect.x=0;
	rect.y=0;
	rect.w=SX;
	rect.h=STATUSY;

	SDL_SetClipRect(screen,&rect);

	if (player.livelost) {
		SDL_FillRect(screen,NULL,SDL_MapRGB(screen->format,255,255,255));
		player.livelost=false;
	} else {
		SDL_FillRect(screen,NULL,SDL_MapRGB(screen->format,50,0,0));
	}


	SDL_Surface *tmp;
	SDL_Color color;
	color.r=255; color.g=255; color.b=255;

	std::ostringstream sstr;

	sstr << " Enemylines  Space = Fire, B = Clear, Return = Restart    ";
	sstr<< "   Lives: " << player.lives << "   Score: " << player.score << "  Level: " << player.level;


#ifdef USE_TTF
	tmp=TTF_RenderText_Blended(font,sstr.str().c_str(), color );
	if (!tmp) return;

	SDL_BlitSurface(tmp,NULL,screen,NULL);
	SDL_FreeSurface(tmp);
#else 
	write(screen,rect,sstr.str().c_str());

#endif

}



void collide() {
	std::vector <Entity>::iterator it,cit;

	for (it=entities.begin();it!=entities.end();it++) {
		if ((*it).collisionid!=1) continue;
		if ((*it).collision!=0) continue;
		for (cit=entities.begin();cit!=entities.end();cit++) {
			if ((*cit).collisionid!=0) continue;
			if ((*cit).collision!=0) continue;

			if (abs(int((*cit).x-(*it).x))>HIT) continue;
			if (abs(int((*cit).y-(*it).y))>HIT) continue;


			(*it).collision=5;
			(*cit).collision=5;
		}

	}
}


int main() {
   std::cout<<std::endl<<std::endl<<"enemy lines" << std::endl;
   std::cout<<"\t Copyright (C) 2005 Raphael Pikrin " << std::endl;
   std::cout<<"\t http://raum1.memebot.com/enemylines/" << std::endl << std::endl;
   std::cout<<"This program is free software; you can redistribute it and/or"<<std::endl;
   std::cout<<"modify it under the terms of the GNU General Public License," <<std::endl;
   std::cout<<"version 2, as published by the Free Software Foundation; You"<<std::endl;
   std::cout<<"should have received a copy of the GNU General Public License"<<std::endl;
   std::cout<<"along with this program; if not, write to the Free Software" <<std::endl;
   std::cout<<"Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA."<<std::endl;
   std::cout << std::endl << std::endl;



	if(SDL_Init(SDL_INIT_VIDEO)==-1) {
		std::cerr <<"SDL_Init error SDL_INIT_VIDEO" << std::endl;
		return EXIT_FAILURE;
	}
	if(!(screen=SDL_SetVideoMode(SX,SY,0,0))) {
		std::cerr <<"SDL_SetVideoMode error" << std::endl;
		return EXIT_FAILURE;
	}

#ifdef USE_TTF
	if(TTF_Init()==-1) {
		std::cerr <<"TTF_Init error" << std::endl;
		return EXIT_FAILURE;
	}

	font=TTF_OpenFont("data/font.ttf", 10);
	if (!font) {
		std::cerr <<"TTF_OpenFont error data/font.ttf" << std::endl;
		return EXIT_FAILURE;
	}
#endif
	

	if (!res::load()) {
		std::cerr <<"res::load error" << std::endl;
		return EXIT_FAILURE;
	}
	rg.seed();

	setup();
	start();




   SDL_Event event;
	Uint8 *keys;
	int timestamp,oldtimestamp;
   while (true) {
	


      if (SDL_PollEvent(&event)) {
         switch (event.type) {
				case SDL_QUIT: return 0; break;
            case SDL_KEYDOWN:
					switch (event.key.keysym.sym) {
						case SDLK_SPACE: 
							fire(MSHOT);
							break;
						case SDLK_b: fire(MBLANK1); break;
						case SDLK_a:
							add_entities(true);
							break;
						case SDLK_RETURN:
							start();
							break;
						default:
							//std::cout << "keydown " << std::endl;
							break;
					}
               break;

			}
		}

		if (player.gameover) {


		} else {
			add_entities();


			int diff;
			timestamp=SDL_GetTicks();
			diff=timestamp-oldtimestamp;
			oldtimestamp=timestamp;

			keys = SDL_GetKeyState(NULL);

			if (!player.gameover) {
				if (keys[SDLK_UP]) {player.y-=0.2f*diff;}
				if (keys[SDLK_DOWN]) {player.y+=0.2f*diff;}

				if (player.y<-50) { player.y=-50; }
				if (player.y>SY+50) { player.y=SY+50; }
			}

			std::vector <Entity>::iterator it;
			std::vector <std::vector <Entity>::iterator> toremove;
			std::vector <std::vector <Entity>::iterator>::iterator rit;

			for (it=entities.begin();it!=entities.end();it++) {
				(*it).draw(screen);

				for (int i=0;i<diff;i++) {
					(*it).tick(diff);
				}
				if (!(*it).remove) continue;

				if ((*it).through) { player.lives--; player.livelost=true; }
				if ((*it).collision==1) {
					player.score+=10;
				}

				toremove.push_back(it);
			}
			player.draw(screen);
	
			for (rit=toremove.begin();rit!=toremove.end();rit++) {
				entities.erase((*rit));
			}

			collide();
			round++;

			if (player.lives<0) player.gameover=true;
		}

		status();

		SDL_Flip(screen);
		SDL_Delay(3);

	}
}

}

int main(int argc, char ** argv) {
	el::main();
}
